/*
 *  mixer.c
 *  $Id: mixer.c 1.4 1999/08/20 01:15:57 frago Exp frago $
 *
 *  Digital "elevon" mixer program for pulse proportional Radio Control.
 *  Inputs: RB0=aileron and RB1=elevator channels
 *  Outputs: RB2=left and RB3=right elevon channels
 *
 *  Revision history:
 *    990724 Francis Gormarker, initial issue.
 *
 *    990729 FrGo, swapped inputs so RB0 is elevator, RB1 aileron.
 *           Added output pulse width limits.
 *
 *    990820 FrGo, optimized measure and emit loops for 10 us resolution.
 *           Added sampling jitter filter.
 */

const char *RCS_ID = "$Id: mixer.c 1.4 1999/08/20 01:15:57 frago Exp frago $";

char n0 = 0;
char n1 = 0;
char nn0 = 0;
char nn1 = 0;
char last0 = 0;
char last1 = 0;
char y0 = 0;
char y1 = 0;
char count = 0;
char state = 0;


/* Wait approx "t" ms */
void delay(char t)
{
  char i;

  while (t != 0) {
    for (i=0; i<100; i++) {
      clear_wdt();
    }
    t--;
  }
}

/*
 *  Measure high pulse width on RB0 and RB1.
 *  Counter loop takes 10 us (4MHz Xtal) per iteration.
 *  Standard servo timing gives:
 *    Upper limit  2.0 ms => 200
 *    Center value 1.5 ms => 150
 *    Lower limit  1.0 ms => 100
 */
void measure(void)
{
  //output_high_port_b(4);
  n0 = 0;
  n1 = 0;
  //while (input_pin_port_b(0) != 0) {
  while (input_pin_port_b(0)) {
  }
  while (input_pin_port_b(0) == 0) {
  //while (!input_pin_port_b(0)) {
    clear_wdt();
  }
  //while (input_pin_port_b(0) != 0) {
  while (input_pin_port_b(0)) {
    n0++;
  }


  while (input_pin_port_b(1) == 0) {
  //while (!input_pin_port_b(1)) {
    clear_wdt();
  }
  //while (input_pin_port_b(1) != 0) {
  while (input_pin_port_b(1)) {
    n1++;
  }
  //output_low_port_b(4);
}


void emit(void)
{
  char t;

  t = y0;
  output_high_port_b(2);
  //while (t != 0) {
  while (t) {
    t--;
  }
  output_low_port_b(2);

  t = y1;
  output_high_port_b(3);
  //while (t != 0) {
  while (t) {
    t--;
  }
  output_low_port_b(3);
}


/* Toggle output RB5 in endless loop */
main()
{
  char ch;
  char delta,tmp;

  set_bit( STATUS, RP0 );
  OPTION_REG = 7;    //set prescaler to 1:256
  set_tris_b(0x03);  // RB0-RB1 inputs, RB2-RB7 outputs
  set_tris_a(0xff);  // RA0-3 inputs
  clear_bit( STATUS, RP0 );
  output_port_b( 0 );

  //enable_interrupt( GIE );
  //enable_interrupt( T0IE );  //enable TMR0 overflow bit

  /* Move servos to center position */
  y0 = 150;
  y1 = 150;
  emit();

  /* Flash LEDs */
  output_high_port_b(4);
  delay(150);
  emit();
  output_low_port_b(4);
  output_high_port_b(5);
  delay(150);
  emit();
  output_high_port_b(4);
  output_low_port_b(5);
  delay(150);
  emit();
  output_low_port_b(4);
  output_low_port_b(5);


  state = 0;
  while (1) {
    clear_wdt();
    measure();
    clear_wdt();

    /* Elevator sampling jitter filter */
    delta = n0 - nn0;
    tmp = n0;
    if (delta == 1) {
       n0 = last0;
    } else if (delta == 255) {
       n0 = last0;
    }
    nn0 = tmp;
    last0 = n0;

    /* Aileron sampling jitter filter */
    delta = n1 - nn1;
    tmp = n1;
    if (delta == 1) {
       n1 = last1;
    } else if (delta == 255) {
       n1 = last1;
    }
    nn1 = tmp;
    last1 = n1;


    //y0 = n0 + (n1 - 107);
    //y1 = n0 + (107 - n1);
    y0 = n1 + (n0 - 150);
    y1 = n1 + (150 - n0);
    if (y0 < 100) {
       y0 = 100;
    } else if (y0 > 200) {
       y0 = 200;
    }
    if (y1 < 100) {
       y1 = 100;
    } else if (y1 > 200) {
       y1 = 200;
    }
    emit();

    if (n0 > 150) {
       output_high_port_b(5);
    }
    state++;
    if (state == 255) {
       output_high_port_b(4);
    } else {
       output_low_port_b(4);
    }
    output_low_port_b(5);
  }
}

